# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
ShapeExtend module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_shapeextend.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ShapeExtend
else:
    import _ShapeExtend

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _ShapeExtend.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _ShapeExtend.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _ShapeExtend.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _ShapeExtend.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _ShapeExtend.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _ShapeExtend.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _ShapeExtend.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _ShapeExtend.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _ShapeExtend.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _ShapeExtend.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _ShapeExtend.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _ShapeExtend.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _ShapeExtend.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _ShapeExtend.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _ShapeExtend.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _ShapeExtend.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _ShapeExtend.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _ShapeExtend:
_ShapeExtend.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _ShapeExtend.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TopoDS
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp
import OCC.Core.Geom
import OCC.Core.GeomAbs
import OCC.Core.TColgp
import OCC.Core.TColGeom
import OCC.Core.TopTools

from enum import IntEnum
from OCC.Core.Exception import *

ShapeExtend_OK = _ShapeExtend.ShapeExtend_OK
ShapeExtend_DONE1 = _ShapeExtend.ShapeExtend_DONE1
ShapeExtend_DONE2 = _ShapeExtend.ShapeExtend_DONE2
ShapeExtend_DONE3 = _ShapeExtend.ShapeExtend_DONE3
ShapeExtend_DONE4 = _ShapeExtend.ShapeExtend_DONE4
ShapeExtend_DONE5 = _ShapeExtend.ShapeExtend_DONE5
ShapeExtend_DONE6 = _ShapeExtend.ShapeExtend_DONE6
ShapeExtend_DONE7 = _ShapeExtend.ShapeExtend_DONE7
ShapeExtend_DONE8 = _ShapeExtend.ShapeExtend_DONE8
ShapeExtend_DONE = _ShapeExtend.ShapeExtend_DONE
ShapeExtend_FAIL1 = _ShapeExtend.ShapeExtend_FAIL1
ShapeExtend_FAIL2 = _ShapeExtend.ShapeExtend_FAIL2
ShapeExtend_FAIL3 = _ShapeExtend.ShapeExtend_FAIL3
ShapeExtend_FAIL4 = _ShapeExtend.ShapeExtend_FAIL4
ShapeExtend_FAIL5 = _ShapeExtend.ShapeExtend_FAIL5
ShapeExtend_FAIL6 = _ShapeExtend.ShapeExtend_FAIL6
ShapeExtend_FAIL7 = _ShapeExtend.ShapeExtend_FAIL7
ShapeExtend_FAIL8 = _ShapeExtend.ShapeExtend_FAIL8
ShapeExtend_FAIL = _ShapeExtend.ShapeExtend_FAIL
ShapeExtend_Natural = _ShapeExtend.ShapeExtend_Natural
ShapeExtend_Uniform = _ShapeExtend.ShapeExtend_Uniform
ShapeExtend_Unitary = _ShapeExtend.ShapeExtend_Unitary


class ShapeExtend_Status(IntEnum):
	ShapeExtend_OK = 0
	ShapeExtend_DONE1 = 1
	ShapeExtend_DONE2 = 2
	ShapeExtend_DONE3 = 3
	ShapeExtend_DONE4 = 4
	ShapeExtend_DONE5 = 5
	ShapeExtend_DONE6 = 6
	ShapeExtend_DONE7 = 7
	ShapeExtend_DONE8 = 8
	ShapeExtend_DONE = 9
	ShapeExtend_FAIL1 = 10
	ShapeExtend_FAIL2 = 11
	ShapeExtend_FAIL3 = 12
	ShapeExtend_FAIL4 = 13
	ShapeExtend_FAIL5 = 14
	ShapeExtend_FAIL6 = 15
	ShapeExtend_FAIL7 = 16
	ShapeExtend_FAIL8 = 17
	ShapeExtend_FAIL = 18
ShapeExtend_OK = ShapeExtend_Status.ShapeExtend_OK
ShapeExtend_DONE1 = ShapeExtend_Status.ShapeExtend_DONE1
ShapeExtend_DONE2 = ShapeExtend_Status.ShapeExtend_DONE2
ShapeExtend_DONE3 = ShapeExtend_Status.ShapeExtend_DONE3
ShapeExtend_DONE4 = ShapeExtend_Status.ShapeExtend_DONE4
ShapeExtend_DONE5 = ShapeExtend_Status.ShapeExtend_DONE5
ShapeExtend_DONE6 = ShapeExtend_Status.ShapeExtend_DONE6
ShapeExtend_DONE7 = ShapeExtend_Status.ShapeExtend_DONE7
ShapeExtend_DONE8 = ShapeExtend_Status.ShapeExtend_DONE8
ShapeExtend_DONE = ShapeExtend_Status.ShapeExtend_DONE
ShapeExtend_FAIL1 = ShapeExtend_Status.ShapeExtend_FAIL1
ShapeExtend_FAIL2 = ShapeExtend_Status.ShapeExtend_FAIL2
ShapeExtend_FAIL3 = ShapeExtend_Status.ShapeExtend_FAIL3
ShapeExtend_FAIL4 = ShapeExtend_Status.ShapeExtend_FAIL4
ShapeExtend_FAIL5 = ShapeExtend_Status.ShapeExtend_FAIL5
ShapeExtend_FAIL6 = ShapeExtend_Status.ShapeExtend_FAIL6
ShapeExtend_FAIL7 = ShapeExtend_Status.ShapeExtend_FAIL7
ShapeExtend_FAIL8 = ShapeExtend_Status.ShapeExtend_FAIL8
ShapeExtend_FAIL = ShapeExtend_Status.ShapeExtend_FAIL

class ShapeExtend_Parametrisation(IntEnum):
	ShapeExtend_Natural = 0
	ShapeExtend_Uniform = 1
	ShapeExtend_Unitary = 2
ShapeExtend_Natural = ShapeExtend_Parametrisation.ShapeExtend_Natural
ShapeExtend_Uniform = ShapeExtend_Parametrisation.ShapeExtend_Uniform
ShapeExtend_Unitary = ShapeExtend_Parametrisation.ShapeExtend_Unitary


def Handle_ShapeExtend_BasicMsgRegistrator_Create() -> "opencascade::handle< ShapeExtend_BasicMsgRegistrator >":
    return _ShapeExtend.Handle_ShapeExtend_BasicMsgRegistrator_Create()

def Handle_ShapeExtend_BasicMsgRegistrator_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeExtend_BasicMsgRegistrator >":
    return _ShapeExtend.Handle_ShapeExtend_BasicMsgRegistrator_DownCast(t)

def Handle_ShapeExtend_BasicMsgRegistrator_IsNull(t: "opencascade::handle< ShapeExtend_BasicMsgRegistrator > const &") -> "bool":
    return _ShapeExtend.Handle_ShapeExtend_BasicMsgRegistrator_IsNull(t)

def Handle_ShapeExtend_ComplexCurve_Create() -> "opencascade::handle< ShapeExtend_ComplexCurve >":
    return _ShapeExtend.Handle_ShapeExtend_ComplexCurve_Create()

def Handle_ShapeExtend_ComplexCurve_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeExtend_ComplexCurve >":
    return _ShapeExtend.Handle_ShapeExtend_ComplexCurve_DownCast(t)

def Handle_ShapeExtend_ComplexCurve_IsNull(t: "opencascade::handle< ShapeExtend_ComplexCurve > const &") -> "bool":
    return _ShapeExtend.Handle_ShapeExtend_ComplexCurve_IsNull(t)

def Handle_ShapeExtend_CompositeSurface_Create() -> "opencascade::handle< ShapeExtend_CompositeSurface >":
    return _ShapeExtend.Handle_ShapeExtend_CompositeSurface_Create()

def Handle_ShapeExtend_CompositeSurface_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeExtend_CompositeSurface >":
    return _ShapeExtend.Handle_ShapeExtend_CompositeSurface_DownCast(t)

def Handle_ShapeExtend_CompositeSurface_IsNull(t: "opencascade::handle< ShapeExtend_CompositeSurface > const &") -> "bool":
    return _ShapeExtend.Handle_ShapeExtend_CompositeSurface_IsNull(t)

def Handle_ShapeExtend_WireData_Create() -> "opencascade::handle< ShapeExtend_WireData >":
    return _ShapeExtend.Handle_ShapeExtend_WireData_Create()

def Handle_ShapeExtend_WireData_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeExtend_WireData >":
    return _ShapeExtend.Handle_ShapeExtend_WireData_DownCast(t)

def Handle_ShapeExtend_WireData_IsNull(t: "opencascade::handle< ShapeExtend_WireData > const &") -> "bool":
    return _ShapeExtend.Handle_ShapeExtend_WireData_IsNull(t)

def Handle_ShapeExtend_MsgRegistrator_Create() -> "opencascade::handle< ShapeExtend_MsgRegistrator >":
    return _ShapeExtend.Handle_ShapeExtend_MsgRegistrator_Create()

def Handle_ShapeExtend_MsgRegistrator_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeExtend_MsgRegistrator >":
    return _ShapeExtend.Handle_ShapeExtend_MsgRegistrator_DownCast(t)

def Handle_ShapeExtend_MsgRegistrator_IsNull(t: "opencascade::handle< ShapeExtend_MsgRegistrator > const &") -> "bool":
    return _ShapeExtend.Handle_ShapeExtend_MsgRegistrator_IsNull(t)
class ShapeExtend_DataMapOfShapeListOfMsg(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_List< Message_Msg >,TopTools_ShapeMapHasher >::iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_begin(self)

    def end(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_List< Message_Msg >,TopTools_ShapeMapHasher >::iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_end(self)

    def cbegin(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_List< Message_Msg >,TopTools_ShapeMapHasher >::const_iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_cbegin(self)

    def cend(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_List< Message_Msg >,TopTools_ShapeMapHasher >::const_iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_cend(self)

    def __init__(self, *args):
        _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_swiginit(self, _ShapeExtend.new_ShapeExtend_DataMapOfShapeListOfMsg(*args))

    def Exchange(self, theOther: "ShapeExtend_DataMapOfShapeListOfMsg") -> "void":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Exchange(self, theOther)

    def Assign(self, theOther: "ShapeExtend_DataMapOfShapeListOfMsg") -> "NCollection_DataMap< TopoDS_Shape,Message_ListOfMsg,TopTools_ShapeMapHasher > &":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Assign(self, theOther)

    def Set(self, theOther: "ShapeExtend_DataMapOfShapeListOfMsg") -> "NCollection_DataMap< TopoDS_Shape,Message_ListOfMsg,TopTools_ShapeMapHasher > &":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_ReSize(self, N)

    def Bind(self, theKey: "TopoDS_Shape", theItem: "Message_ListOfMsg") -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Bind(self, theKey, theItem)

    def Bound(self, theKey: "TopoDS_Shape", theItem: "Message_ListOfMsg") -> "NCollection_List< Message_Msg > *":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "TopoDS_Shape") -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_IsBound(self, theKey)

    def UnBind(self, theKey: "TopoDS_Shape") -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_UnBind(self, theKey)

    def Seek(self, theKey: "TopoDS_Shape") -> "NCollection_List< Message_Msg > const *":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Find(self, *args)

    def ChangeSeek(self, theKey: "TopoDS_Shape") -> "NCollection_List< Message_Msg > *":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "TopoDS_Shape") -> "NCollection_List< Message_Msg > &":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_ChangeFind(self, theKey)

    def __call__(self, *args) -> "NCollection_List< Message_Msg > &":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Clear(self, *args)
    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_DataMapOfShapeListOfMsg

    def Size(self) -> "Standard_Integer":
        return _ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_Size(self)

# Register ShapeExtend_DataMapOfShapeListOfMsg in _ShapeExtend:
_ShapeExtend.ShapeExtend_DataMapOfShapeListOfMsg_swigregister(ShapeExtend_DataMapOfShapeListOfMsg)

class ShapeExtend_DataMapOfTransientListOfMsg(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< opencascade::handle< Standard_Transient >,NCollection_List< Message_Msg >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_begin(self)

    def end(self) -> "NCollection_DataMap< opencascade::handle< Standard_Transient >,NCollection_List< Message_Msg >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_end(self)

    def cbegin(self) -> "NCollection_DataMap< opencascade::handle< Standard_Transient >,NCollection_List< Message_Msg >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::const_iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_cbegin(self)

    def cend(self) -> "NCollection_DataMap< opencascade::handle< Standard_Transient >,NCollection_List< Message_Msg >,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::const_iterator":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_cend(self)

    def __init__(self, *args):
        _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_swiginit(self, _ShapeExtend.new_ShapeExtend_DataMapOfTransientListOfMsg(*args))

    def Exchange(self, theOther: "ShapeExtend_DataMapOfTransientListOfMsg") -> "void":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Exchange(self, theOther)

    def Assign(self, theOther: "ShapeExtend_DataMapOfTransientListOfMsg") -> "NCollection_DataMap< opencascade::handle< Standard_Transient >,Message_ListOfMsg,TColStd_MapTransientHasher > &":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Assign(self, theOther)

    def Set(self, theOther: "ShapeExtend_DataMapOfTransientListOfMsg") -> "NCollection_DataMap< opencascade::handle< Standard_Transient >,Message_ListOfMsg,TColStd_MapTransientHasher > &":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_ReSize(self, N)

    def Bind(self, theKey: "opencascade::handle< Standard_Transient > const &", theItem: "Message_ListOfMsg") -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Bind(self, theKey, theItem)

    def Bound(self, theKey: "opencascade::handle< Standard_Transient > const &", theItem: "Message_ListOfMsg") -> "NCollection_List< Message_Msg > *":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "opencascade::handle< Standard_Transient > const &") -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_IsBound(self, theKey)

    def UnBind(self, theKey: "opencascade::handle< Standard_Transient > const &") -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_UnBind(self, theKey)

    def Seek(self, theKey: "opencascade::handle< Standard_Transient > const &") -> "NCollection_List< Message_Msg > const *":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Find(self, *args)

    def ChangeSeek(self, theKey: "opencascade::handle< Standard_Transient > const &") -> "NCollection_List< Message_Msg > *":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "opencascade::handle< Standard_Transient > const &") -> "NCollection_List< Message_Msg > &":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_ChangeFind(self, theKey)

    def __call__(self, *args) -> "NCollection_List< Message_Msg > &":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Clear(self, *args)
    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_DataMapOfTransientListOfMsg

    def Size(self) -> "Standard_Integer":
        return _ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_Size(self)

# Register ShapeExtend_DataMapOfTransientListOfMsg in _ShapeExtend:
_ShapeExtend.ShapeExtend_DataMapOfTransientListOfMsg_swigregister(ShapeExtend_DataMapOfTransientListOfMsg)

class shapeextend(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def DecodeStatus(*args) -> "Standard_Boolean":
        r"""
        Tells if a bit flag contains bit corresponding to enumerated status.

        Parameters
        ----------
        flag: int
        status: ShapeExtend_Status

        Returns
        -------
        bool

        """
        return _ShapeExtend.shapeextend_DecodeStatus(*args)

    @staticmethod
    def EncodeStatus(*args) -> "Standard_Integer":
        r"""
        Encodes status (enumeration) to a bit flag.

        Parameters
        ----------
        status: ShapeExtend_Status

        Returns
        -------
        int

        """
        return _ShapeExtend.shapeextend_EncodeStatus(*args)

    @staticmethod
    def Init(*args) -> "void":
        r"""
        Inits using of shapeextend. currently, loads messages output by shapehealing algorithms.

        Returns
        -------
        None

        """
        return _ShapeExtend.shapeextend_Init(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _ShapeExtend.shapeextend_swiginit(self, _ShapeExtend.new_shapeextend())
    __swig_destroy__ = _ShapeExtend.delete_shapeextend

# Register shapeextend in _ShapeExtend:
_ShapeExtend.shapeextend_swigregister(shapeextend)

def shapeextend_DecodeStatus(*args) -> "Standard_Boolean":
    r"""
    Tells if a bit flag contains bit corresponding to enumerated status.

    Parameters
    ----------
    flag: int
    status: ShapeExtend_Status

    Returns
    -------
    bool

    """
    return _ShapeExtend.shapeextend_DecodeStatus(*args)

def shapeextend_EncodeStatus(*args) -> "Standard_Integer":
    r"""
    Encodes status (enumeration) to a bit flag.

    Parameters
    ----------
    status: ShapeExtend_Status

    Returns
    -------
    int

    """
    return _ShapeExtend.shapeextend_EncodeStatus(*args)

def shapeextend_Init(*args) -> "void":
    r"""
    Inits using of shapeextend. currently, loads messages output by shapehealing algorithms.

    Returns
    -------
    None

    """
    return _ShapeExtend.shapeextend_Init(*args)

class ShapeExtend_BasicMsgRegistrator(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Empty constructor.

        Returns
        -------
        None

        """
        _ShapeExtend.ShapeExtend_BasicMsgRegistrator_swiginit(self, _ShapeExtend.new_ShapeExtend_BasicMsgRegistrator(*args))

    def Send(self, *args) -> "void":
        r"""
        Sends a message to be attached to the object. object can be of any type interpreted by redefined msgregistrator.

        Parameters
        ----------
        object: Standard_Transient
        message: Message_Msg
        gravity: Message_Gravity

        Returns
        -------
        None
        Sends a message to be attached to the shape.

        Parameters
        ----------
        shape: TopoDS_Shape
        message: Message_Msg
        gravity: Message_Gravity

        Returns
        -------
        None
        Calls send method with null transient.

        Parameters
        ----------
        message: Message_Msg
        gravity: Message_Gravity

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_BasicMsgRegistrator_Send(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeExtend_BasicMsgRegistrator_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_BasicMsgRegistrator

# Register ShapeExtend_BasicMsgRegistrator in _ShapeExtend:
_ShapeExtend.ShapeExtend_BasicMsgRegistrator_swigregister(ShapeExtend_BasicMsgRegistrator)

class ShapeExtend_ComplexCurve(OCC.Core.Geom.Geom_Curve):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def CheckConnectivity(self, *args) -> "Standard_Boolean":
        r"""
        Checks geometrical connectivity of the curves, including closure (sets fields myclosed).

        Parameters
        ----------
        Preci: float

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_ComplexCurve_CheckConnectivity(self, *args)

    def Curve(self, *args) -> "opencascade::handle< Geom_Curve > const &":
        r"""
        Returns curve given by its index.

        Parameters
        ----------
        index: int

        Returns
        -------
        opencascade::handle<Geom_Curve>

        """
        return _ShapeExtend.ShapeExtend_ComplexCurve_Curve(self, *args)

    def GetScaleFactor(self, *args) -> "Standard_Real":
        r"""
        Returns scale factor for recomputing of deviatives.

        Parameters
        ----------
        ind: int

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_ComplexCurve_GetScaleFactor(self, *args)

    def LocalToGlobal(self, *args) -> "Standard_Real":
        r"""
        Returns global parameter for the whole curve according to the segment and local parameter on it.

        Parameters
        ----------
        index: int
        Ulocal: float

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_ComplexCurve_LocalToGlobal(self, *args)

    def LocateParameter(self, *args) -> "Standard_Integer":
        r"""
        Returns number of the curve for the given parameter u and local paramete r uout for the found curve.

        Parameters
        ----------
        U: float

        Returns
        -------
        UOut: float

        """
        return _ShapeExtend.ShapeExtend_ComplexCurve_LocateParameter(self, *args)

    def NbCurves(self, *args) -> "Standard_Integer":
        r"""
        Returns number of curves.

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_ComplexCurve_NbCurves(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeExtend_ComplexCurve_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_ComplexCurve

# Register ShapeExtend_ComplexCurve in _ShapeExtend:
_ShapeExtend.ShapeExtend_ComplexCurve_swigregister(ShapeExtend_ComplexCurve)

class ShapeExtend_CompositeSurface(OCC.Core.Geom.Geom_Surface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Empty constructor.

        Returns
        -------
        None
        Initializes by a grid of surfaces (calls init()).

        Parameters
        ----------
        GridSurf: TColGeom_HArray2OfSurface
        param: ShapeExtend_Parametrisation,optional
        	default value is ShapeExtend_Natural

        Returns
        -------
        None
        Initializes by a grid of surfaces (calls init()).

        Parameters
        ----------
        GridSurf: TColGeom_HArray2OfSurface
        UJoints: TColStd_Array1OfReal
        VJoints: TColStd_Array1OfReal

        Returns
        -------
        None

        """
        _ShapeExtend.ShapeExtend_CompositeSurface_swiginit(self, _ShapeExtend.new_ShapeExtend_CompositeSurface(*args))

    def CheckConnectivity(self, *args) -> "Standard_Boolean":
        r"""
        Checks geometrical connectivity of the patches, including closedness (sets fields muuclosed and myvclosed).

        Parameters
        ----------
        prec: float

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_CheckConnectivity(self, *args)

    def ComputeJointValues(self, *args) -> "void":
        r"""
        Computes joint values according to parameter.

        Parameters
        ----------
        param: ShapeExtend_Parametrisation,optional
        	default value is ShapeExtend_Natural

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_ComputeJointValues(self, *args)

    def GlobalToLocal(self, *args) -> "gp_Pnt2d":
        r"""
        Converts global parameters uv to local parameters uv on patch i,j.

        Parameters
        ----------
        i: int
        j: int
        UV: gp_Pnt2d

        Returns
        -------
        gp_Pnt2d

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_GlobalToLocal(self, *args)

    def GlobalToLocalTransformation(self, *args) -> "Standard_Boolean":
        r"""
        Computes transformation operator and ufactor descrinbing affine transformation required to convert global parameters on composite surface to local parameters on patch (i,j): uv = ( ufactor, 1. ) x trsf * uv; note: thus trsf contains shift and scale by v, scale by u is stored in ufact. returns true if transformation is not an identity.

        Parameters
        ----------
        i: int
        j: int
        Trsf: gp_Trsf2d

        Returns
        -------
        uFact: float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_GlobalToLocalTransformation(self, *args)

    def Init(self, *args) -> "Standard_Boolean":
        r"""
        Initializes by a grid of surfaces. all the surfaces of the grid must have geometrical connectivity as stated above. if geometrical connectivity is not satisfied, method returns false. however, class is initialized even in that case. //! last parameter defines how global parametrisation (joint values) will be computed: shapeextend_natural: u1 = u11min, ui+1 = ui + (ui1max-ui1min), etc. shapeextend_uniform: ui = i-1, vj = j-1 shapeextend_unitary: ui = (i-1)/nu, vi = (j-1)/nv.

        Parameters
        ----------
        GridSurf: TColGeom_HArray2OfSurface
        param: ShapeExtend_Parametrisation,optional
        	default value is ShapeExtend_Natural

        Returns
        -------
        bool
        Initializes by a grid of surfaces with given global parametrisation defined by ujoints and vjoints arrays, each having langth equal to number of patches in corresponding direction + 1. global joint values should be sorted in increasing order. all the surfaces of the grid must have geometrical connectivity as stated above. if geometrical connectivity is not satisfied, method returns false. however, class is initialized even in that case.

        Parameters
        ----------
        GridSurf: TColGeom_HArray2OfSurface
        UJoints: TColStd_Array1OfReal
        VJoints: TColStd_Array1OfReal

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_Init(self, *args)

    def LocalToGlobal(self, *args) -> "gp_Pnt2d":
        r"""
        Converts local parameters uv on patch i,j to global parameters uv.

        Parameters
        ----------
        i: int
        j: int
        uv: gp_Pnt2d

        Returns
        -------
        gp_Pnt2d

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_LocalToGlobal(self, *args)

    def LocateUParameter(self, *args) -> "Standard_Integer":
        r"""
        Returns number of col that contains given (global) parameter.

        Parameters
        ----------
        U: float

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_LocateUParameter(self, *args)

    def LocateUVPoint(self, *args) -> "void":
        r"""
        Returns number of row and col of surface that contains given point.

        Parameters
        ----------
        pnt: gp_Pnt2d

        Returns
        -------
        i: int
        j: int

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_LocateUVPoint(self, *args)

    def LocateVParameter(self, *args) -> "Standard_Integer":
        r"""
        Returns number of row that contains given (global) parameter.

        Parameters
        ----------
        V: float

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_LocateVParameter(self, *args)

    def NbUPatches(self, *args) -> "Standard_Integer":
        r"""
        Returns number of patches in u direction.

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_NbUPatches(self, *args)

    def NbVPatches(self, *args) -> "Standard_Integer":
        r"""
        Returns number of patches in v direction.

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_NbVPatches(self, *args)

    def Patch(self, *args) -> "opencascade::handle< Geom_Surface > const &":
        r"""
        Returns one surface patch.

        Parameters
        ----------
        i: int
        j: int

        Returns
        -------
        opencascade::handle<Geom_Surface>
        Returns one surface patch that contains given (global) parameters.

        Parameters
        ----------
        U: float
        V: float

        Returns
        -------
        opencascade::handle<Geom_Surface>
        Returns one surface patch that contains given point.

        Parameters
        ----------
        pnt: gp_Pnt2d

        Returns
        -------
        opencascade::handle<Geom_Surface>

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_Patch(self, *args)

    def Patches(self, *args) -> "opencascade::handle< TColGeom_HArray2OfSurface > const &":
        r"""
        Returns grid of surfaces.

        Returns
        -------
        opencascade::handle<TColGeom_HArray2OfSurface>

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_Patches(self, *args)

    def SetUFirstValue(self, *args) -> "void":
        r"""
        Changes starting value for global u parametrisation (all other joint values are shifted accordingly).

        Parameters
        ----------
        UFirst: float

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_SetUFirstValue(self, *args)

    def SetUJointValues(self, *args) -> "Standard_Boolean":
        r"""
        Sets the array of u values corresponding to joint points, which define global parametrisation of the surface. number of values in array should be equal to nbupatches()+1. all the values should be sorted in increasing order. if this is not satisfied, does nothing and returns false.

        Parameters
        ----------
        UJoints: TColStd_Array1OfReal

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_SetUJointValues(self, *args)

    def SetVFirstValue(self, *args) -> "void":
        r"""
        Changes starting value for global v parametrisation (all other joint values are shifted accordingly).

        Parameters
        ----------
        VFirst: float

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_SetVFirstValue(self, *args)

    def SetVJointValues(self, *args) -> "Standard_Boolean":
        r"""
        Sets the array of v values corresponding to joint points, which define global parametrisation of the surface number of values in array should be equal to nbvpatches()+1. all the values should be sorted in increasing order. if this is not satisfied, does nothing and returns false.

        Parameters
        ----------
        VJoints: TColStd_Array1OfReal

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_SetVJointValues(self, *args)

    def UGlobalToLocal(self, *args) -> "Standard_Real":
        r"""
        Converts global parameter u to local parameter u on patch i,j.

        Parameters
        ----------
        i: int
        j: int
        U: float

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_UGlobalToLocal(self, *args)

    def UJointValue(self, *args) -> "Standard_Real":
        r"""
        Returns i-th joint value in u direction (1-st is global umin, (nbupatches()+1)-th is global umax on the composite surface).

        Parameters
        ----------
        i: int

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_UJointValue(self, *args)

    def UJointValues(self, *args) -> "opencascade::handle< TColStd_HArray1OfReal >":
        r"""
        Returns the array of u values corresponding to joint points between patches as well as to start and end points, which define global parametrisation of the surface.

        Returns
        -------
        opencascade::handle<TColStd_HArray1OfReal>

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_UJointValues(self, *args)

    def ULocalToGlobal(self, *args) -> "Standard_Real":
        r"""
        Converts local parameter u on patch i,j to global parameter u.

        Parameters
        ----------
        i: int
        j: int
        u: float

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_ULocalToGlobal(self, *args)

    def VGlobalToLocal(self, *args) -> "Standard_Real":
        r"""
        Converts global parameter v to local parameter v on patch i,j.

        Parameters
        ----------
        i: int
        j: int
        V: float

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_VGlobalToLocal(self, *args)

    def VJointValue(self, *args) -> "Standard_Real":
        r"""
        Returns j-th joint value in v direction (1-st is global vmin, (nbvpatches()+1)-th is global vmax on the composite surface).

        Parameters
        ----------
        j: int

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_VJointValue(self, *args)

    def VJointValues(self, *args) -> "opencascade::handle< TColStd_HArray1OfReal >":
        r"""
        Returns the array of v values corresponding to joint points between patches as well as to start and end points, which define global parametrisation of the surface.

        Returns
        -------
        opencascade::handle<TColStd_HArray1OfReal>

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_VJointValues(self, *args)

    def VLocalToGlobal(self, *args) -> "Standard_Real":
        r"""
        Converts local parameter v on patch i,j to global parameter v.

        Parameters
        ----------
        i: int
        j: int
        v: float

        Returns
        -------
        float

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_VLocalToGlobal(self, *args)

    def Value(self, *args) -> "gp_Pnt":
        r"""
        Computes the point of parameter pnt on the grid.

        Parameters
        ----------
        pnt: gp_Pnt2d

        Returns
        -------
        gp_Pnt

        """
        return _ShapeExtend.ShapeExtend_CompositeSurface_Value(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeExtend_CompositeSurface_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_CompositeSurface

# Register ShapeExtend_CompositeSurface in _ShapeExtend:
_ShapeExtend.ShapeExtend_CompositeSurface_swigregister(ShapeExtend_CompositeSurface)

class ShapeExtend_Explorer(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an object explorer.

        Returns
        -------
        None

        """
        _ShapeExtend.ShapeExtend_Explorer_swiginit(self, _ShapeExtend.new_ShapeExtend_Explorer(*args))

    def CompoundFromSeq(self, *args) -> "TopoDS_Shape":
        r"""
        Converts a sequence of shapes to a compound.

        Parameters
        ----------
        seqval: TopTools_HSequenceOfShape

        Returns
        -------
        TopoDS_Shape

        """
        return _ShapeExtend.ShapeExtend_Explorer_CompoundFromSeq(self, *args)

    def DispatchList(self, *args) -> "void":
        r"""
        Dispatches starting list of shapes according to their type, to the appropriate resulting lists for each of these lists, if it is null, it is firstly created else, new items are appended to the already existing ones.

        Parameters
        ----------
        list: TopTools_HSequenceOfShape
        vertices: TopTools_HSequenceOfShape
        edges: TopTools_HSequenceOfShape
        wires: TopTools_HSequenceOfShape
        faces: TopTools_HSequenceOfShape
        shells: TopTools_HSequenceOfShape
        solids: TopTools_HSequenceOfShape
        compsols: TopTools_HSequenceOfShape
        compounds: TopTools_HSequenceOfShape

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_Explorer_DispatchList(self, *args)

    def ListFromSeq(self, *args) -> "void":
        r"""
        Converts a sequence of shapes to a list of shapes <clear> if true (d), commands the list to start from scratch else, the list is cumulated.

        Parameters
        ----------
        seqval: TopTools_HSequenceOfShape
        lisval: TopTools_ListOfShape
        clear: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_Explorer_ListFromSeq(self, *args)

    def SeqFromCompound(self, *args) -> "opencascade::handle< TopTools_HSequenceOfShape >":
        r"""
        Converts a compound to a list of shapes if <comp> is not a compound, the list contains only <comp> if <comp> is null, the list is empty if <comp> is a compound, its sub-shapes are put into the list then if <expcomp> is true, if a sub-shape is a compound, it is not put to the list but its sub-shapes are (recursive).

        Parameters
        ----------
        comp: TopoDS_Shape
        expcomp: bool

        Returns
        -------
        opencascade::handle<TopTools_HSequenceOfShape>

        """
        return _ShapeExtend.ShapeExtend_Explorer_SeqFromCompound(self, *args)

    def SeqFromList(self, *args) -> "opencascade::handle< TopTools_HSequenceOfShape >":
        r"""
        Converts a list of shapes to a sequence of shapes.

        Parameters
        ----------
        lisval: TopTools_ListOfShape

        Returns
        -------
        opencascade::handle<TopTools_HSequenceOfShape>

        """
        return _ShapeExtend.ShapeExtend_Explorer_SeqFromList(self, *args)

    def ShapeType(self, *args) -> "TopAbs_ShapeEnum":
        r"""
        Returns the type of a shape: true type if <compound> is false if <compound> is true and <shape> is a compound, iterates on its items. if all are of the same type, returns this type. else, returns compound. if it is empty, returns shape for a null shape, returns shape.

        Parameters
        ----------
        shape: TopoDS_Shape
        compound: bool

        Returns
        -------
        TopAbs_ShapeEnum

        """
        return _ShapeExtend.ShapeExtend_Explorer_ShapeType(self, *args)

    def SortedCompound(self, *args) -> "TopoDS_Shape":
        r"""
        Builds a compound from the given shape. it explores the shape level by level, according to the <explore> argument. if <explore> is false, only compound items are explored, else all items are. the following shapes are added to resulting compound: - shapes which comply to <type> - if <type> is wire, considers also free edges (and makes wires) - if <type> is shell, considers also free faces (and makes shells) if <compound> is true, gathers items in compounds which correspond to starting compound,solid or shell containers, or items directly contained in a compound.

        Parameters
        ----------
        shape: TopoDS_Shape
        type: TopAbs_ShapeEnum
        explore: bool
        compound: bool

        Returns
        -------
        TopoDS_Shape

        """
        return _ShapeExtend.ShapeExtend_Explorer_SortedCompound(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_Explorer

# Register ShapeExtend_Explorer in _ShapeExtend:
_ShapeExtend.ShapeExtend_Explorer_swigregister(ShapeExtend_Explorer)

class ShapeExtend_WireData(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Empty constructor, creates empty wire with no edges.

        Returns
        -------
        None
        Constructor initializing the data from topods_wire. calls init(wire,chained).

        Parameters
        ----------
        wire: TopoDS_Wire
        chained: bool,optional
        	default value is Standard_True
        theManifoldMode: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        _ShapeExtend.ShapeExtend_WireData_swiginit(self, _ShapeExtend.new_ShapeExtend_WireData(*args))

    def Add(self, *args) -> "void":
        r"""
        Adds an edge to a wire, being defined (not yet ended) this is the plain, basic, function to add an edge <num> = 0 (d): appends at end <num> = 1: preprends at start else, insert before <num> remark : null edge is simply ignored.

        Parameters
        ----------
        edge: TopoDS_Edge
        atnum: int,optional
        	default value is 0

        Returns
        -------
        None
        Adds an entire wire, considered as a list of edges remark : the wire is assumed to be ordered (topods_iterator is used).

        Parameters
        ----------
        wire: TopoDS_Wire
        atnum: int,optional
        	default value is 0

        Returns
        -------
        None
        Adds a wire in the form of wiredata.

        Parameters
        ----------
        wire: ShapeExtend_WireData
        atnum: int,optional
        	default value is 0

        Returns
        -------
        None
        Adds an edge or a wire invoking corresponding method add.

        Parameters
        ----------
        shape: TopoDS_Shape
        atnum: int,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_Add(self, *args)

    def AddOriented(self, *args) -> "void":
        r"""
        Adds an edge to start or end of <self>, according to <mode> 0: at end, as direct 1: at end, as reversed 2: at start, as direct 3: at start, as reversed < 0: no adding.

        Parameters
        ----------
        edge: TopoDS_Edge
        mode: int

        Returns
        -------
        None
        Adds a wire to start or end of <self>, according to <mode> 0: at end, as direct 1: at end, as reversed 2: at start, as direct 3: at start, as reversed < 0: no adding.

        Parameters
        ----------
        wire: TopoDS_Wire
        mode: int

        Returns
        -------
        None
        Adds an edge or a wire invoking corresponding method addoriented.

        Parameters
        ----------
        shape: TopoDS_Shape
        mode: int

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_AddOriented(self, *args)

    def Clear(self, *args) -> "void":
        r"""
        Clears data about wire.

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_Clear(self, *args)

    def ComputeSeams(self, *args) -> "void":
        r"""
        Computes the list of seam edges by default (direct call), computing is enforced for indirect call (from isseam) it is redone only if not yet already done or if the list of edges has changed remark : a seam edge is an edge present twice in the list, once as forward and once as reversed each sense has its own pcurve, the one for forward must be set in first.

        Parameters
        ----------
        enforce: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_ComputeSeams(self, *args)

    def Edge(self, *args) -> "TopoDS_Edge":
        r"""
        Returns <num>th edge.

        Parameters
        ----------
        num: int

        Returns
        -------
        TopoDS_Edge

        """
        return _ShapeExtend.ShapeExtend_WireData_Edge(self, *args)

    def Index(self, *args) -> "Standard_Integer":
        r"""
        Returns the index of the edge if the edge is a seam the orientation is also checked returns 0 if the edge is not found in the list.

        Parameters
        ----------
        edge: TopoDS_Edge

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_WireData_Index(self, *args)

    def Init(self, *args) -> "Standard_Boolean":
        r"""
        Copies data from another wiredata.

        Parameters
        ----------
        other: ShapeExtend_WireData

        Returns
        -------
        None
        Loads an already existing wire if <chained> is true (default), edges are added in the sequence as they are explored by topods_iterator else, if <chained> is false, wire is explored by breptools_wireexplorer and it is guaranteed that edges will be sequencially connected. remark : in the latter case it can happen that not all edges will be found (because of limitations of breptools_wireexplorer for disconnected wires and wires with seam edges).

        Parameters
        ----------
        wire: TopoDS_Wire
        chained: bool,optional
        	default value is Standard_True
        theManifoldMode: bool,optional
        	default value is Standard_True

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_WireData_Init(self, *args)

    def IsSeam(self, *args) -> "Standard_Boolean":
        r"""
        Tells if an edge is seam (see computeseams) an edge is considered as seam if it presents twice in the edge list, once as forward and once as reversed.

        Parameters
        ----------
        num: int

        Returns
        -------
        bool

        """
        return _ShapeExtend.ShapeExtend_WireData_IsSeam(self, *args)

    def GetManifoldMode(self) -> "Standard_Boolean":
        r"""GetManifoldMode(ShapeExtend_WireData self) -> Standard_Boolean"""
        return _ShapeExtend.ShapeExtend_WireData_GetManifoldMode(self)

    def SetManifoldMode(self, value: "Standard_Boolean") -> "void":
        r"""SetManifoldMode(ShapeExtend_WireData self, Standard_Boolean value)"""
        return _ShapeExtend.ShapeExtend_WireData_SetManifoldMode(self, value)

    def NbEdges(self, *args) -> "Standard_Integer":
        r"""
        Returns the count of currently recorded edges.

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_WireData_NbEdges(self, *args)

    def NbNonManifoldEdges(self, *args) -> "Standard_Integer":
        r"""
        Returns the count of currently recorded non-manifold edges.

        Returns
        -------
        int

        """
        return _ShapeExtend.ShapeExtend_WireData_NbNonManifoldEdges(self, *args)

    def NonmanifoldEdge(self, *args) -> "TopoDS_Edge":
        r"""
        Returns <num>th nonmanifold edge.

        Parameters
        ----------
        num: int

        Returns
        -------
        TopoDS_Edge

        """
        return _ShapeExtend.ShapeExtend_WireData_NonmanifoldEdge(self, *args)

    def NonmanifoldEdges(self, *args) -> "opencascade::handle< TopTools_HSequenceOfShape >":
        r"""
        Returns sequence of non-manifold edges this sequence can be not empty if wire data set in manifold mode but initial wire has internal orientation or contains internal edges.

        Returns
        -------
        opencascade::handle<TopTools_HSequenceOfShape>

        """
        return _ShapeExtend.ShapeExtend_WireData_NonmanifoldEdges(self, *args)

    def Remove(self, *args) -> "void":
        r"""
        Removes an edge, given its rank. by default removes the last edge.

        Parameters
        ----------
        num: int,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_Remove(self, *args)

    def Reverse(self, *args) -> "void":
        r"""
        Reverses the sense of the list and the orientation of each edge this method should be called when either wire has no seam edges or face is not available.

        Returns
        -------
        None
        Reverses the sense of the list and the orientation of each edge the face is necessary for swapping pcurves for seam edges (first pcurve corresponds to orientation forward, and second to reversed; when edge is reversed, pcurves must be swapped) if face is null, no swapping is performed.

        Parameters
        ----------
        face: TopoDS_Face

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_Reverse(self, *args)

    def Set(self, *args) -> "void":
        r"""
        Replaces an edge at the given rank number <num> with new one. default is last edge (<num> = 0).

        Parameters
        ----------
        edge: TopoDS_Edge
        num: int,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_Set(self, *args)

    def SetDegeneratedLast(self, *args) -> "void":
        r"""
        When the wire contains at least one degenerated edge, sets it as last one note : it is useful to process pcurves, for instance, while the pcurve of a dgnr may not be computed from its 3d part (there is none) it is computed after the other edges have been computed and chained.

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_SetDegeneratedLast(self, *args)

    def SetLast(self, *args) -> "void":
        r"""
        Does a circular permutation in order to set <num>th edge last.

        Parameters
        ----------
        num: int

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_WireData_SetLast(self, *args)

    def Wire(self, *args) -> "TopoDS_Wire":
        r"""
        Makes topods_wire using brep_builder (just creates the topods_wire object and adds all edges into it). this method should be called when the wire is correct (for example, after successful fixes by shapefix_wire) and adjacent edges share common vertices. in case if adjacent edges do not share the same vertices the resulting topods_wire will be invalid.

        Returns
        -------
        TopoDS_Wire

        """
        return _ShapeExtend.ShapeExtend_WireData_Wire(self, *args)

    def WireAPIMake(self, *args) -> "TopoDS_Wire":
        r"""
        Makes topods_wire using brepapi_makewire. class brepapi_makewire merges geometrically coincided vertices and can disturb correct order of edges in the wire. if this class fails, null shape is returned.

        Returns
        -------
        TopoDS_Wire

        """
        return _ShapeExtend.ShapeExtend_WireData_WireAPIMake(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeExtend_WireData_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_WireData

# Register ShapeExtend_WireData in _ShapeExtend:
_ShapeExtend.ShapeExtend_WireData_swigregister(ShapeExtend_WireData)

class ShapeExtend_MsgRegistrator(ShapeExtend_BasicMsgRegistrator):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an object.

        Returns
        -------
        None

        """
        _ShapeExtend.ShapeExtend_MsgRegistrator_swiginit(self, _ShapeExtend.new_ShapeExtend_MsgRegistrator(*args))

    def MapShape(self, *args) -> "ShapeExtend_DataMapOfShapeListOfMsg const &":
        r"""
        Returns a map of shapes and message list.

        Returns
        -------
        ShapeExtend_DataMapOfShapeListOfMsg

        """
        return _ShapeExtend.ShapeExtend_MsgRegistrator_MapShape(self, *args)

    def MapTransient(self, *args) -> "ShapeExtend_DataMapOfTransientListOfMsg const &":
        r"""
        Returns a map of objects and message list.

        Returns
        -------
        ShapeExtend_DataMapOfTransientListOfMsg

        """
        return _ShapeExtend.ShapeExtend_MsgRegistrator_MapTransient(self, *args)

    def Send(self, *args) -> "void":
        r"""
        Sends a message to be attached to the object. if the object is in the map then the message is added to the list, otherwise the object is firstly added to the map.

        Parameters
        ----------
        object: Standard_Transient
        message: Message_Msg
        gravity: Message_Gravity

        Returns
        -------
        None
        Sends a message to be attached to the shape. if the shape is in the map then the message is added to the list, otherwise the shape is firstly added to the map.

        Parameters
        ----------
        shape: TopoDS_Shape
        message: Message_Msg
        gravity: Message_Gravity

        Returns
        -------
        None

        """
        return _ShapeExtend.ShapeExtend_MsgRegistrator_Send(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeExtend_MsgRegistrator_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeExtend.delete_ShapeExtend_MsgRegistrator

# Register ShapeExtend_MsgRegistrator in _ShapeExtend:
_ShapeExtend.ShapeExtend_MsgRegistrator_swigregister(ShapeExtend_MsgRegistrator)








from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.gp import *
from OCC.Core.TopAbs import *


class TopCnx_EdgeFaceTransition:
	def __init__(self) -> None: ...
	def AddInterference(self, Tole: float, Tang: gp_Dir, Norm: gp_Dir, Curv: float, Or: TopAbs_Orientation, Tr: TopAbs_Orientation, BTr: TopAbs_Orientation) -> None: ...
	def BoundaryTransition(self) -> TopAbs_Orientation: ...
	@overload
	def Reset(self, Tgt: gp_Dir, Norm: gp_Dir, Curv: float) -> None: ...
	@overload
	def Reset(self, Tgt: gp_Dir) -> None: ...
	def Transition(self) -> TopAbs_Orientation: ...

# harray1 classes
# harray2 classes
# hsequence classes




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
BinDrivers module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_bindrivers.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _BinDrivers
else:
    import _BinDrivers

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _BinDrivers.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _BinDrivers.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BinDrivers.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BinDrivers.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _BinDrivers.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _BinDrivers.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _BinDrivers.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _BinDrivers.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _BinDrivers.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _BinDrivers.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BinDrivers.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _BinDrivers.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _BinDrivers.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BinDrivers.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BinDrivers.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BinDrivers.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _BinDrivers.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _BinDrivers:
_BinDrivers.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _BinDrivers.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.BinMDF
import OCC.Core.TDF
import OCC.Core.BinObjMgt
import OCC.Core.Storage
import OCC.Core.TDocStd
import OCC.Core.CDF
import OCC.Core.CDM
import OCC.Core.Resource
import OCC.Core.PCDM
import OCC.Core.BinLDrivers

from enum import IntEnum
from OCC.Core.Exception import *

BinDrivers_ENDATTRLIST = _BinDrivers.BinDrivers_ENDATTRLIST
BinDrivers_ENDLABEL = _BinDrivers.BinDrivers_ENDLABEL


class BinDrivers_Marker(IntEnum):
	BinDrivers_ENDATTRLIST = - 1
	BinDrivers_ENDLABEL = - 2
BinDrivers_ENDATTRLIST = BinDrivers_Marker.BinDrivers_ENDATTRLIST
BinDrivers_ENDLABEL = BinDrivers_Marker.BinDrivers_ENDLABEL


def Handle_BinDrivers_DocumentRetrievalDriver_Create() -> "opencascade::handle< BinDrivers_DocumentRetrievalDriver >":
    return _BinDrivers.Handle_BinDrivers_DocumentRetrievalDriver_Create()

def Handle_BinDrivers_DocumentRetrievalDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< BinDrivers_DocumentRetrievalDriver >":
    return _BinDrivers.Handle_BinDrivers_DocumentRetrievalDriver_DownCast(t)

def Handle_BinDrivers_DocumentRetrievalDriver_IsNull(t: "opencascade::handle< BinDrivers_DocumentRetrievalDriver > const &") -> "bool":
    return _BinDrivers.Handle_BinDrivers_DocumentRetrievalDriver_IsNull(t)

def Handle_BinDrivers_DocumentStorageDriver_Create() -> "opencascade::handle< BinDrivers_DocumentStorageDriver >":
    return _BinDrivers.Handle_BinDrivers_DocumentStorageDriver_Create()

def Handle_BinDrivers_DocumentStorageDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< BinDrivers_DocumentStorageDriver >":
    return _BinDrivers.Handle_BinDrivers_DocumentStorageDriver_DownCast(t)

def Handle_BinDrivers_DocumentStorageDriver_IsNull(t: "opencascade::handle< BinDrivers_DocumentStorageDriver > const &") -> "bool":
    return _BinDrivers.Handle_BinDrivers_DocumentStorageDriver_IsNull(t)
class bindrivers(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def AttributeDrivers(*args) -> "opencascade::handle< BinMDF_ADriverTable >":
        r"""
        Creates the table of drivers of types supported.

        Parameters
        ----------
        MsgDrv: Message_Messenger

        Returns
        -------
        opencascade::handle<BinMDF_ADriverTable>

        """
        return _BinDrivers.bindrivers_AttributeDrivers(*args)

    @staticmethod
    def DefineFormat(*args) -> "void":
        r"""
        Defines format 'binocaf' and registers its read and write drivers in the specified application.

        Parameters
        ----------
        theApp: TDocStd_Application

        Returns
        -------
        None

        """
        return _BinDrivers.bindrivers_DefineFormat(*args)

    @staticmethod
    def Factory(*args) -> "opencascade::handle< Standard_Transient > const &":
        r"""
        No available documentation.

        Parameters
        ----------
        theGUID: Standard_GUID

        Returns
        -------
        opencascade::handle<Standard_Transient>

        """
        return _BinDrivers.bindrivers_Factory(*args)

    @staticmethod
    def StorageVersion(*args) -> "TCollection_AsciiString":
        r"""
        Returns '1'.

        Returns
        -------
        TCollection_AsciiString

        """
        return _BinDrivers.bindrivers_StorageVersion(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _BinDrivers.bindrivers_swiginit(self, _BinDrivers.new_bindrivers())
    __swig_destroy__ = _BinDrivers.delete_bindrivers

# Register bindrivers in _BinDrivers:
_BinDrivers.bindrivers_swigregister(bindrivers)

def bindrivers_AttributeDrivers(*args) -> "opencascade::handle< BinMDF_ADriverTable >":
    r"""
    Creates the table of drivers of types supported.

    Parameters
    ----------
    MsgDrv: Message_Messenger

    Returns
    -------
    opencascade::handle<BinMDF_ADriverTable>

    """
    return _BinDrivers.bindrivers_AttributeDrivers(*args)

def bindrivers_DefineFormat(*args) -> "void":
    r"""
    Defines format 'binocaf' and registers its read and write drivers in the specified application.

    Parameters
    ----------
    theApp: TDocStd_Application

    Returns
    -------
    None

    """
    return _BinDrivers.bindrivers_DefineFormat(*args)

def bindrivers_Factory(*args) -> "opencascade::handle< Standard_Transient > const &":
    r"""
    No available documentation.

    Parameters
    ----------
    theGUID: Standard_GUID

    Returns
    -------
    opencascade::handle<Standard_Transient>

    """
    return _BinDrivers.bindrivers_Factory(*args)

def bindrivers_StorageVersion(*args) -> "TCollection_AsciiString":
    r"""
    Returns '1'.

    Returns
    -------
    TCollection_AsciiString

    """
    return _BinDrivers.bindrivers_StorageVersion(*args)

class BinDrivers_DocumentRetrievalDriver(OCC.Core.BinLDrivers.BinLDrivers_DocumentRetrievalDriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructor.

        Returns
        -------
        None

        """
        _BinDrivers.BinDrivers_DocumentRetrievalDriver_swiginit(self, _BinDrivers.new_BinDrivers_DocumentRetrievalDriver(*args))

    def Clear(self, *args) -> "void":
        r"""
        Clears the namedshape driver.

        Returns
        -------
        None

        """
        return _BinDrivers.BinDrivers_DocumentRetrievalDriver_Clear(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_BinDrivers_DocumentRetrievalDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _BinDrivers.delete_BinDrivers_DocumentRetrievalDriver

# Register BinDrivers_DocumentRetrievalDriver in _BinDrivers:
_BinDrivers.BinDrivers_DocumentRetrievalDriver_swigregister(BinDrivers_DocumentRetrievalDriver)

class BinDrivers_DocumentStorageDriver(OCC.Core.BinLDrivers.BinLDrivers_DocumentStorageDriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructor.

        Returns
        -------
        None

        """
        _BinDrivers.BinDrivers_DocumentStorageDriver_swiginit(self, _BinDrivers.new_BinDrivers_DocumentStorageDriver(*args))

    def IsWithTriangles(self, *args) -> "Standard_Boolean":
        r"""
        Return true if shape should be stored with triangles.

        Returns
        -------
        bool

        """
        return _BinDrivers.BinDrivers_DocumentStorageDriver_IsWithTriangles(self, *args)

    def SetWithTriangles(self, *args) -> "void":
        r"""
        Set if triangulation should be stored or not.

        Parameters
        ----------
        theMessageDriver: Message_Messenger
        theWithTriangulation: bool

        Returns
        -------
        None

        """
        return _BinDrivers.BinDrivers_DocumentStorageDriver_SetWithTriangles(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_BinDrivers_DocumentStorageDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _BinDrivers.delete_BinDrivers_DocumentStorageDriver

# Register BinDrivers_DocumentStorageDriver in _BinDrivers:
_BinDrivers.BinDrivers_DocumentStorageDriver_swigregister(BinDrivers_DocumentStorageDriver)








""" A flask webserver. """

import sys
import uuid

from OCC.Display.WebGl.threejs_renderer import ThreejsRenderer, OCC_VERSION, \
        THREEJS_RELEASE, color_to_hex, export_edgedata_to_json, spinning_cursor
from OCC.Extend.TopologyUtils import is_edge, is_wire, discretize_edge, discretize_wire
from OCC.Core.Tesselator import ShapeTesselator
# Import following for building vertex (or point cloud) in WebGL
from OCC.Core.gp import gp_Pnt
from OCC.Core.BRep import BRep_Builder
from OCC.Core.TopoDS import TopoDS_Compound
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_MakeVertex

from flask import Flask, render_template


def format_color(r, g, b):
    return '0x%02x%02x%02x' % (r, g, b)


class RenderWraper(ThreejsRenderer):
    def __init__(self, path=None,
            default_shape_color=format_color(166, 166, 166), # light grey
            default_edge_color=format_color(32, 32, 32), # dark grey
            default_vertex_color=format_color(8, 8, 8)): # darker gray
        super().__init__(path)
        self._3js_vertex = {}
        self._default_shape_color = default_shape_color
        self._default_edge_color = default_edge_color
        self._default_vertex_color = default_vertex_color

    def ConvertShape(self,
                     shape,
                     export_edges=False,
                     color=(0.65, 0.65, 0.7),
                     specular_color=(0.2, 0.2, 0.2),
                     shininess=0.9,
                     transparency=0.,
                     line_color=(0, 0., 0.),
                     line_width=1.,
                     point_size=1.,
                     mesh_quality=1.):
        # if the shape is an edge or a wire, use the related functions
        color = color_to_hex(color)
        specular_color = color_to_hex(specular_color)
        if is_edge(shape):
            print("discretize an edge")
            pnts = discretize_edge(shape)
            edge_hash = "edg%s" % uuid.uuid4().hex
            shape_content = export_edgedata_to_json(edge_hash, pnts)
            # store this edge hash
            self._3js_edges[edge_hash] = [color, line_width, shape_content]
            return self._3js_shapes, self._3js_edges, self._3js_vertex
        elif is_wire(shape):
            print("discretize a wire")
            pnts = discretize_wire(shape)
            wire_hash = "wir%s" % uuid.uuid4().hex
            shape_content = export_edgedata_to_json(wire_hash, pnts)
            # store this edge hash
            self._3js_edges[wire_hash] = [color, line_width, shape_content]
            return self._3js_shapes, self._3js_edges, self._3js_vertex
        # if shape is array of gp_Pnt
        elif isinstance(shape, list) and isinstance(shape[0], gp_Pnt):
            print("storage points")
            vertices_list = []  # will be passed to javascript
            BB = BRep_Builder()
            compound = TopoDS_Compound()
            BB.MakeCompound(compound)

            for vertex in shape:
                vertext_to_add = BRepBuilderAPI_MakeVertex(vertex).Shape()
                BB.Add(compound, vertext_to_add)
                vertices_list.append([vertex.X(), vertex.Y(), vertex.Z()])
            points_hash = "pnt%s" % uuid.uuid4().hex
            # store this vertex hash. Note: TopoDS_Compound did not save now
            self._3js_vertex[points_hash] = [color, point_size, vertices_list]
            return self._3js_shapes, self._3js_edges, self._3js_vertex

        # convert as TopoDS_Shape
        shape_uuid = uuid.uuid4().hex
        shape_hash = "shp%s" % shape_uuid
        # tesselate
        tess = ShapeTesselator(shape)
        tess.Compute(compute_edges=export_edges,
                     mesh_quality=mesh_quality,
                     parallel=True)
        # update spinning cursor
        sys.stdout.write("\r%s mesh shape %s, %i triangles     " % (next(self.spinning_cursor),
                                                                    shape_hash,
                                                                    tess.ObjGetTriangleCount()))
        sys.stdout.flush()
        # export to 3JS
        # generate the mesh
        shape_content = tess.ExportShapeToThreejsJSONString(shape_uuid)
        # add this shape to the shape dict, sotres everything related to it
        self._3js_shapes[shape_hash] = [export_edges, color, specular_color, shininess, transparency,
                                        line_color, line_width, shape_content]
        # draw edges if necessary
        if export_edges:
            # export each edge to a single json
            # get number of edges
            nbr_edges = tess.ObjGetEdgeCount()
            for i_edge in range(nbr_edges):
                # after that, the file can be appended
                edge_content = ''
                edge_point_set = []
                nbr_vertices = tess.ObjEdgeGetVertexCount(i_edge)
                for i_vert in range(nbr_vertices):
                    edge_point_set.append(tess.GetEdgeVertex(i_edge, i_vert))
                # write to file
                edge_hash = "edg%s" % uuid.uuid4().hex
                edge_content += export_edgedata_to_json(edge_hash, edge_point_set)
                # store this edge hash, with black color
                self._3js_edges[edge_hash] = [color_to_hex((0, 0, 0)), line_width, edge_content]
        return self._3js_shapes, self._3js_edges, self._3js_vertex


class RenderConfig(object):
    def __init__(self, bg_gradient_color1="#ced7de", bg_gradient_color2="#808080",
                 vertex_shader=None, fragment_shader=None, uniforms=None):
        self._occ_version = OCC_VERSION
        self._3js_version = THREEJS_RELEASE
        self._bg_gradient_color1 = bg_gradient_color1
        self._bg_gradient_color2 = bg_gradient_color2
        self._vertex_shader = vertex_shader
        self._fragment_shader = fragment_shader
        self._uniforms = uniforms


app = Flask(__name__)
my_ren = RenderWraper()
render_cfg = RenderConfig()


if __name__ == '__main__':
    @app.route('/')
    @app.route('/index')
    def index():
        """PythonOCC Demo Page"""
        # remove shapes from previous (avoid duplicate shape after F5 refresh)
        my_ren._3js_shapes={}
        my_ren._3js_edges={}
        my_ren._3js_vertex={}

        # import additional modules for building a box and a torus.
        from OCC.Core.BRepPrimAPI import BRepPrimAPI_MakeBox, BRepPrimAPI_MakeTorus
        from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_Transform
        from OCC.Core.gp import gp_Trsf
        import time


        def translate_shp(shp, vec, copy=False):
            trns = gp_Trsf()
            trns.SetTranslation(vec)
            brep_trns = BRepBuilderAPI_Transform(shp, trns, copy)
            brep_trns.Build()
            return brep_trns.Shape()
        box = BRepPrimAPI_MakeBox(100., 200., 300.).Shape()
        torus = BRepPrimAPI_MakeTorus(300., 105).Shape()
        t_torus = translate_shp(torus, gp_Vec(700, 0, 0))


        init_time = time.time()
        my_ren.ConvertShape(box, export_edges=True)
        my_ren.ConvertShape(t_torus, export_edges=True)
        final_time = time.time()
        print("\nTotal meshing time : ", final_time - init_time)

        return render_template('index.html', occ_version=OCC_VERSION, threejs_version=THREEJS_RELEASE,
                               render_cfg=render_cfg, occ_shapes=my_ren._3js_shapes, occ_edges=my_ren._3js_edges,
                               occ_vertex=my_ren._3js_vertex)

    app.run(host='localhost', port=8080, debug=False)



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.OSD import *
from OCC.Core.Message import *
from OCC.Core.Poly import *


class rwstl:
	@staticmethod
	def ReadAscii(thePath: OSD_Path, theProgress: Optional[Message_ProgressRange] = Message_ProgressRange()) -> Poly_Triangulation: ...
	@staticmethod
	def ReadBinary(thePath: OSD_Path, theProgress: Optional[Message_ProgressRange] = Message_ProgressRange()) -> Poly_Triangulation: ...
	@overload
	@staticmethod
	def ReadFile(theFile: OSD_Path, aProgInd: Optional[Message_ProgressRange] = Message_ProgressRange()) -> Poly_Triangulation: ...
	@overload
	@staticmethod
	def ReadFile(theFile: str, aProgInd: Optional[Message_ProgressRange] = Message_ProgressRange()) -> Poly_Triangulation: ...
	@staticmethod
	def WriteAscii(theMesh: Poly_Triangulation, thePath: OSD_Path, theProgress: Optional[Message_ProgressRange] = Message_ProgressRange()) -> bool: ...
	@staticmethod
	def WriteBinary(theMesh: Poly_Triangulation, thePath: OSD_Path, theProgress: Optional[Message_ProgressRange] = Message_ProgressRange()) -> bool: ...

#classnotwrapped
class RWStl_Reader: ...

# harray1 classes
# harray2 classes
# hsequence classes

rwstl_ReadAscii = rwstl.ReadAscii
rwstl_ReadBinary = rwstl.ReadBinary
rwstl_ReadFile = rwstl.ReadFile
rwstl_ReadFile = rwstl.ReadFile
rwstl_WriteAscii = rwstl.WriteAscii
rwstl_WriteBinary = rwstl.WriteBinary



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.AppParCurves import *
from OCC.Core.math import *


class AppCont_Function:
	def FirstParameter(self) -> float: ...
	def GetNbOf2dPoints(self) -> int: ...
	def GetNbOf3dPoints(self) -> int: ...
	def GetNumberOfPoints(self) -> Tuple[int, int]: ...
	def LastParameter(self) -> float: ...

class AppCont_LeastSquare:
	def __init__(self, SSP: AppCont_Function, U0: float, U1: float, FirstCons: AppParCurves_Constraint, LastCons: AppParCurves_Constraint, Deg: int, NbPoints: int) -> None: ...
	def Error(self) -> Tuple[float, float, float]: ...
	def IsDone(self) -> bool: ...
	def Value(self) -> AppParCurves_MultiCurve: ...

# harray1 classes
# harray2 classes
# hsequence classes




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
Precision module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_precision.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _Precision
else:
    import _Precision

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _Precision.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _Precision.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _Precision.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _Precision.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _Precision.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _Precision.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _Precision.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _Precision.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _Precision.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _Precision.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _Precision.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _Precision.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _Precision.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _Precision.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _Precision.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _Precision.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _Precision.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _Precision:
_Precision.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _Precision.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection

from enum import IntEnum
from OCC.Core.Exception import *



class precision(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def Angular(*args) -> "Standard_Real":
        r"""
        Returns the recommended precision value when checking the equality of two angles (given in radians). standard_real angle1 = ... , angle2 = ... ; if ( abs( angle2 - angle1 ) < precision::angular() ) ... the tolerance of angular equality may be used to check the parallelism of two vectors : gp_vec v1, v2 ; v1 = ... v2 = ... if ( v1.isparallel (v2, precision::angular() ) ) ... the tolerance of angular equality is equal to 1.e-12. note : the tolerance of angular equality can be used when working with scalar products or cross products since sines and angles are equivalent for small angles. therefore, in order to check whether two unit vectors are perpendicular : gp_dir d1, d2 ; d1 = ... d2 = ... you can use : if ( abs( d1.d2 ) < precision::angular() ) ... (although the function isnormal does exist).

        Returns
        -------
        float

        """
        return _Precision.precision_Angular(*args)

    @staticmethod
    def Approximation(*args) -> "Standard_Real":
        r"""
        Returns the precision value in real space, frequently used by approximation algorithms. this function provides an acceptable level of precision for an approximation process to define adjustment limits. the tolerance of approximation is designed to ensure an acceptable computation time when performing an approximation process. that is why the tolerance of approximation is greater than the tolerance of confusion. the tolerance of approximation is equal to : precision::confusion() * 10. (that is, 1.e-6). you may use a smaller tolerance in an approximation algorithm, but this option might be costly.

        Returns
        -------
        float

        """
        return _Precision.precision_Approximation(*args)

    @staticmethod
    def Confusion(*args) -> "Standard_Real":
        r"""
        Returns the recommended precision value when checking coincidence of two points in real space. the tolerance of confusion is used for testing a 3d distance : - two points are considered to be coincident if their distance is smaller than the tolerance of confusion. gp_pnt p1, p2 ; p1 = ... p2 = ... if ( p1.isequal ( p2 , precision::confusion() ) ) then ... - a vector is considered to be null if it has a null length : gp_vec v ; v = ... if ( v.magnitude() < precision::confusion() ) then ... the tolerance of confusion is equal to 1.e-7. the value of the tolerance of confusion is also used to define : - the tolerance of intersection, and - the tolerance of approximation. note : as a rule, coordinate values in cas.cade are not dimensioned, so 1. represents one user unit, whatever value the unit may have : the millimeter, the meter, the inch, or any other unit. let's say that cas.cade algorithms are written to be tuned essentially with mechanical design applications, on the basis of the millimeter. however, these algorithms may be used with any other unit but the tolerance criterion does no longer have the same signification. so pay particular attention to the type of your application, in relation with the impact of your unit on the precision criterion. - for example in mechanical design, if the unit is the millimeter, the tolerance of confusion corresponds to a distance of 1 / 10000 micron, which is rather difficult to measure. - however in other types of applications, such as cartography, where the kilometer is frequently used, the tolerance of confusion corresponds to a greater distance (1 / 10 millimeter). this distance becomes easily measurable, but only within a restricted space which contains some small objects of the complete scene.

        Returns
        -------
        float

        """
        return _Precision.precision_Confusion(*args)

    @staticmethod
    def Infinite(*args) -> "Standard_Real":
        r"""
        Returns a big number that can be considered as infinite. use -infinite() for a negative big number.

        Returns
        -------
        float

        """
        return _Precision.precision_Infinite(*args)

    @staticmethod
    def Intersection(*args) -> "Standard_Real":
        r"""
        Returns the precision value in real space, frequently used by intersection algorithms to decide that a solution is reached. this function provides an acceptable level of precision for an intersection process to define the adjustment limits. the tolerance of intersection is designed to ensure that a point computed by an iterative algorithm as the intersection between two curves is indeed on the intersection. it is obvious that two tangent curves are close to each other, on a large distance. an iterative algorithm of intersection may find points on these curves within the scope of the confusion tolerance, but still far from the true intersection point. in order to force the intersection algorithm to continue the iteration process until a correct point is found on the tangent objects, the tolerance of intersection must be smaller than the tolerance of confusion. on the other hand, the tolerance of intersection must be large enough to minimize the time required by the process to converge to a solution. the tolerance of intersection is equal to : precision::confusion() / 100. (that is, 1.e-9).

        Returns
        -------
        float

        """
        return _Precision.precision_Intersection(*args)

    @staticmethod
    def IsInfinite(*args) -> "Standard_Boolean":
        r"""
        Returns true if r may be considered as an infinite number. currently abs(r) > 1e100.

        Parameters
        ----------
        R: float

        Returns
        -------
        bool

        """
        return _Precision.precision_IsInfinite(*args)

    @staticmethod
    def IsNegativeInfinite(*args) -> "Standard_Boolean":
        r"""
        Returns true if r may be considered as a negative infinite number. currently r < -1e100.

        Parameters
        ----------
        R: float

        Returns
        -------
        bool

        """
        return _Precision.precision_IsNegativeInfinite(*args)

    @staticmethod
    def IsPositiveInfinite(*args) -> "Standard_Boolean":
        r"""
        Returns true if r may be considered as a positive infinite number. currently r > 1e100.

        Parameters
        ----------
        R: float

        Returns
        -------
        bool

        """
        return _Precision.precision_IsPositiveInfinite(*args)

    @staticmethod
    def PApproximation(*args) -> "Standard_Real":
        r"""
        Returns a precision value in parametric space, which may be used by approximation algorithms. the purpose of this function is to provide an acceptable level of precision in parametric space, for an approximation process to define the adjustment limits. the parametric tolerance of approximation is designed to give a mean value in relation with the dimension of the curve or the surface. it considers that a variation of parameter equal to 1. along a curve (or an isoparametric curve of a surface) generates a segment whose length is equal to 100. (default value), or t. the parametric tolerance of intersection is equal to : - precision::approximation() / 100., or precision::approximation() / t.

        Parameters
        ----------
        T: float

        Returns
        -------
        float
        Used for approximations in parametric space on a default curve. //! this is precision::parametric(precision::approximation()).

        Returns
        -------
        float

        """
        return _Precision.precision_PApproximation(*args)

    @staticmethod
    def PConfusion(*args) -> "Standard_Real":
        r"""
        Returns a precision value in parametric space, which may be used : - to test the coincidence of two points in the real space, by using parameter values, or - to test the equality of two parameter values in a parametric space. the parametric tolerance of confusion is designed to give a mean value in relation with the dimension of the curve or the surface. it considers that a variation of parameter equal to 1. along a curve (or an isoparametric curve of a surface) generates a segment whose length is equal to 100. (default value), or t. the parametric tolerance of confusion is equal to : - precision::confusion() / 100., or precision::confusion() / t. the value of the parametric tolerance of confusion is also used to define : - the parametric tolerance of intersection, and - the parametric tolerance of approximation. warning it is rather difficult to define a unique precision value in parametric space. - first consider a curve (c) ; if m is the point of parameter u and m' the point of parameter u+du on the curve, call 'parametric tangent' at point m, for the variation du of the parameter, the quantity : t(u,du)=mm'/du (where mm' represents the distance between the two points m and m', in the real space). - consider the other curve resulting from a scaling transformation of (c) with a scale factor equal to 10. the 'parametric tangent' at the point of parameter u of this curve is ten times greater than the previous one. this shows that for two different curves, the distance between two points on the curve, resulting from the same variation of parameter du, may vary considerably. - moreover, the variation of the parameter along the curve is generally not proportional to the curvilinear abscissa along the curve. so the distance between two points resulting from the same variation of parameter du, at two different points of a curve, may completely differ. - moreover, the parameterization of a surface may generate two quite different 'parametric tangent' values in the u or in the v parametric direction. - last, close to the poles of a sphere (the points which correspond to the values -pi/2. and pi/2. of the v parameter) the u parameter may change from 0 to 2.pi without impacting on the resulting point. therefore, take great care when adjusting a parametric tolerance to your own algorithm.

        Parameters
        ----------
        T: float

        Returns
        -------
        float
        Used to test distances in parametric space on a default curve. //! this is precision::parametric(precision::confusion()).

        Returns
        -------
        float

        """
        return _Precision.precision_PConfusion(*args)

    @staticmethod
    def PIntersection(*args) -> "Standard_Real":
        r"""
        Returns a precision value in parametric space, which may be used by intersection algorithms, to decide that a solution is reached. the purpose of this function is to provide an acceptable level of precision in parametric space, for an intersection process to define the adjustment limits. the parametric tolerance of intersection is designed to give a mean value in relation with the dimension of the curve or the surface. it considers that a variation of parameter equal to 1. along a curve (or an isoparametric curve of a surface) generates a segment whose length is equal to 100. (default value), or t. the parametric tolerance of intersection is equal to : - precision::intersection() / 100., or precision::intersection() / t.

        Parameters
        ----------
        T: float

        Returns
        -------
        float
        Used for intersections in parametric space on a default curve. //! this is precision::parametric(precision::intersection()).

        Returns
        -------
        float

        """
        return _Precision.precision_PIntersection(*args)

    @staticmethod
    def Parametric(*args) -> "Standard_Real":
        r"""
        Convert a real space precision to a parametric space precision. <t> is the mean value of the length of the tangent of the curve or the surface. //! value is p / t.

        Parameters
        ----------
        P: float
        T: float

        Returns
        -------
        float
        Convert a real space precision to a parametric space precision on a default curve. //! value is parametric(p,1.e+2).

        Parameters
        ----------
        P: float

        Returns
        -------
        float

        """
        return _Precision.precision_Parametric(*args)

    @staticmethod
    def SquareConfusion(*args) -> "Standard_Real":
        r"""
        Returns square of confusion. created for speed and convenience.

        Returns
        -------
        float

        """
        return _Precision.precision_SquareConfusion(*args)

    @staticmethod
    def SquarePConfusion(*args) -> "Standard_Real":
        r"""
        Returns square of pconfusion. created for speed and convenience.

        Returns
        -------
        float

        """
        return _Precision.precision_SquarePConfusion(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _Precision.precision_swiginit(self, _Precision.new_precision())
    __swig_destroy__ = _Precision.delete_precision

# Register precision in _Precision:
_Precision.precision_swigregister(precision)

def precision_Angular(*args) -> "Standard_Real":
    r"""
    Returns the recommended precision value when checking the equality of two angles (given in radians). standard_real angle1 = ... , angle2 = ... ; if ( abs( angle2 - angle1 ) < precision::angular() ) ... the tolerance of angular equality may be used to check the parallelism of two vectors : gp_vec v1, v2 ; v1 = ... v2 = ... if ( v1.isparallel (v2, precision::angular() ) ) ... the tolerance of angular equality is equal to 1.e-12. note : the tolerance of angular equality can be used when working with scalar products or cross products since sines and angles are equivalent for small angles. therefore, in order to check whether two unit vectors are perpendicular : gp_dir d1, d2 ; d1 = ... d2 = ... you can use : if ( abs( d1.d2 ) < precision::angular() ) ... (although the function isnormal does exist).

    Returns
    -------
    float

    """
    return _Precision.precision_Angular(*args)

def precision_Approximation(*args) -> "Standard_Real":
    r"""
    Returns the precision value in real space, frequently used by approximation algorithms. this function provides an acceptable level of precision for an approximation process to define adjustment limits. the tolerance of approximation is designed to ensure an acceptable computation time when performing an approximation process. that is why the tolerance of approximation is greater than the tolerance of confusion. the tolerance of approximation is equal to : precision::confusion() * 10. (that is, 1.e-6). you may use a smaller tolerance in an approximation algorithm, but this option might be costly.

    Returns
    -------
    float

    """
    return _Precision.precision_Approximation(*args)

def precision_Confusion(*args) -> "Standard_Real":
    r"""
    Returns the recommended precision value when checking coincidence of two points in real space. the tolerance of confusion is used for testing a 3d distance : - two points are considered to be coincident if their distance is smaller than the tolerance of confusion. gp_pnt p1, p2 ; p1 = ... p2 = ... if ( p1.isequal ( p2 , precision::confusion() ) ) then ... - a vector is considered to be null if it has a null length : gp_vec v ; v = ... if ( v.magnitude() < precision::confusion() ) then ... the tolerance of confusion is equal to 1.e-7. the value of the tolerance of confusion is also used to define : - the tolerance of intersection, and - the tolerance of approximation. note : as a rule, coordinate values in cas.cade are not dimensioned, so 1. represents one user unit, whatever value the unit may have : the millimeter, the meter, the inch, or any other unit. let's say that cas.cade algorithms are written to be tuned essentially with mechanical design applications, on the basis of the millimeter. however, these algorithms may be used with any other unit but the tolerance criterion does no longer have the same signification. so pay particular attention to the type of your application, in relation with the impact of your unit on the precision criterion. - for example in mechanical design, if the unit is the millimeter, the tolerance of confusion corresponds to a distance of 1 / 10000 micron, which is rather difficult to measure. - however in other types of applications, such as cartography, where the kilometer is frequently used, the tolerance of confusion corresponds to a greater distance (1 / 10 millimeter). this distance becomes easily measurable, but only within a restricted space which contains some small objects of the complete scene.

    Returns
    -------
    float

    """
    return _Precision.precision_Confusion(*args)

def precision_Infinite(*args) -> "Standard_Real":
    r"""
    Returns a big number that can be considered as infinite. use -infinite() for a negative big number.

    Returns
    -------
    float

    """
    return _Precision.precision_Infinite(*args)

def precision_Intersection(*args) -> "Standard_Real":
    r"""
    Returns the precision value in real space, frequently used by intersection algorithms to decide that a solution is reached. this function provides an acceptable level of precision for an intersection process to define the adjustment limits. the tolerance of intersection is designed to ensure that a point computed by an iterative algorithm as the intersection between two curves is indeed on the intersection. it is obvious that two tangent curves are close to each other, on a large distance. an iterative algorithm of intersection may find points on these curves within the scope of the confusion tolerance, but still far from the true intersection point. in order to force the intersection algorithm to continue the iteration process until a correct point is found on the tangent objects, the tolerance of intersection must be smaller than the tolerance of confusion. on the other hand, the tolerance of intersection must be large enough to minimize the time required by the process to converge to a solution. the tolerance of intersection is equal to : precision::confusion() / 100. (that is, 1.e-9).

    Returns
    -------
    float

    """
    return _Precision.precision_Intersection(*args)

def precision_IsInfinite(*args) -> "Standard_Boolean":
    r"""
    Returns true if r may be considered as an infinite number. currently abs(r) > 1e100.

    Parameters
    ----------
    R: float

    Returns
    -------
    bool

    """
    return _Precision.precision_IsInfinite(*args)

def precision_IsNegativeInfinite(*args) -> "Standard_Boolean":
    r"""
    Returns true if r may be considered as a negative infinite number. currently r < -1e100.

    Parameters
    ----------
    R: float

    Returns
    -------
    bool

    """
    return _Precision.precision_IsNegativeInfinite(*args)

def precision_IsPositiveInfinite(*args) -> "Standard_Boolean":
    r"""
    Returns true if r may be considered as a positive infinite number. currently r > 1e100.

    Parameters
    ----------
    R: float

    Returns
    -------
    bool

    """
    return _Precision.precision_IsPositiveInfinite(*args)

def precision_PApproximation(*args) -> "Standard_Real":
    r"""
    Returns a precision value in parametric space, which may be used by approximation algorithms. the purpose of this function is to provide an acceptable level of precision in parametric space, for an approximation process to define the adjustment limits. the parametric tolerance of approximation is designed to give a mean value in relation with the dimension of the curve or the surface. it considers that a variation of parameter equal to 1. along a curve (or an isoparametric curve of a surface) generates a segment whose length is equal to 100. (default value), or t. the parametric tolerance of intersection is equal to : - precision::approximation() / 100., or precision::approximation() / t.

    Parameters
    ----------
    T: float

    Returns
    -------
    float
    Used for approximations in parametric space on a default curve. //! this is precision::parametric(precision::approximation()).

    Returns
    -------
    float

    """
    return _Precision.precision_PApproximation(*args)

def precision_PConfusion(*args) -> "Standard_Real":
    r"""
    Returns a precision value in parametric space, which may be used : - to test the coincidence of two points in the real space, by using parameter values, or - to test the equality of two parameter values in a parametric space. the parametric tolerance of confusion is designed to give a mean value in relation with the dimension of the curve or the surface. it considers that a variation of parameter equal to 1. along a curve (or an isoparametric curve of a surface) generates a segment whose length is equal to 100. (default value), or t. the parametric tolerance of confusion is equal to : - precision::confusion() / 100., or precision::confusion() / t. the value of the parametric tolerance of confusion is also used to define : - the parametric tolerance of intersection, and - the parametric tolerance of approximation. warning it is rather difficult to define a unique precision value in parametric space. - first consider a curve (c) ; if m is the point of parameter u and m' the point of parameter u+du on the curve, call 'parametric tangent' at point m, for the variation du of the parameter, the quantity : t(u,du)=mm'/du (where mm' represents the distance between the two points m and m', in the real space). - consider the other curve resulting from a scaling transformation of (c) with a scale factor equal to 10. the 'parametric tangent' at the point of parameter u of this curve is ten times greater than the previous one. this shows that for two different curves, the distance between two points on the curve, resulting from the same variation of parameter du, may vary considerably. - moreover, the variation of the parameter along the curve is generally not proportional to the curvilinear abscissa along the curve. so the distance between two points resulting from the same variation of parameter du, at two different points of a curve, may completely differ. - moreover, the parameterization of a surface may generate two quite different 'parametric tangent' values in the u or in the v parametric direction. - last, close to the poles of a sphere (the points which correspond to the values -pi/2. and pi/2. of the v parameter) the u parameter may change from 0 to 2.pi without impacting on the resulting point. therefore, take great care when adjusting a parametric tolerance to your own algorithm.

    Parameters
    ----------
    T: float

    Returns
    -------
    float
    Used to test distances in parametric space on a default curve. //! this is precision::parametric(precision::confusion()).

    Returns
    -------
    float

    """
    return _Precision.precision_PConfusion(*args)

def precision_PIntersection(*args) -> "Standard_Real":
    r"""
    Returns a precision value in parametric space, which may be used by intersection algorithms, to decide that a solution is reached. the purpose of this function is to provide an acceptable level of precision in parametric space, for an intersection process to define the adjustment limits. the parametric tolerance of intersection is designed to give a mean value in relation with the dimension of the curve or the surface. it considers that a variation of parameter equal to 1. along a curve (or an isoparametric curve of a surface) generates a segment whose length is equal to 100. (default value), or t. the parametric tolerance of intersection is equal to : - precision::intersection() / 100., or precision::intersection() / t.

    Parameters
    ----------
    T: float

    Returns
    -------
    float
    Used for intersections in parametric space on a default curve. //! this is precision::parametric(precision::intersection()).

    Returns
    -------
    float

    """
    return _Precision.precision_PIntersection(*args)

def precision_Parametric(*args) -> "Standard_Real":
    r"""
    Convert a real space precision to a parametric space precision. <t> is the mean value of the length of the tangent of the curve or the surface. //! value is p / t.

    Parameters
    ----------
    P: float
    T: float

    Returns
    -------
    float
    Convert a real space precision to a parametric space precision on a default curve. //! value is parametric(p,1.e+2).

    Parameters
    ----------
    P: float

    Returns
    -------
    float

    """
    return _Precision.precision_Parametric(*args)

def precision_SquareConfusion(*args) -> "Standard_Real":
    r"""
    Returns square of confusion. created for speed and convenience.

    Returns
    -------
    float

    """
    return _Precision.precision_SquareConfusion(*args)

def precision_SquarePConfusion(*args) -> "Standard_Real":
    r"""
    Returns square of pconfusion. created for speed and convenience.

    Returns
    -------
    float

    """
    return _Precision.precision_SquarePConfusion(*args)





